// @ts-ignore
import { debounce } from '@util-lite/debounce';
// @ts-ignore
import { ref, computed, onMounted, onUnmounted, watch, isVue3, } from 'vue-demi';
var visibleKey = isVue3 ? 'visible' : 'value';
export function usePopover(props, className) {
    var _this = this;
    // popover if visible, popover是否可见
    var innerVisibleRef = ref(false);
    // scroll container, 滚动容器
    var scrollElRef = ref(null);
    // timer id for hide popover, 隐藏popover的定时器id
    var timerIdRef = ref(null);
    var debounceUpdatePopperRef = ref(null);
    function cleanup() {
        innerVisibleRef.value = null;
        scrollElRef.value = null;
        timerIdRef.value = null;
        debounceUpdatePopperRef.value = null;
    }
    function genPopoverAttrs(popperClass, opts) {
        var _a;
        if (opts === void 0) { opts = {
            trigger: 'manual',
            placement: 'top',
        }; }
        var popover = props.popover, popoverAttrs = props.popoverAttrs;
        var attrs = Object.assign({
            disabled: !popover && !popoverAttrs.content,
            'popper-class': popperClass,
        }, opts);
        if (typeof popover === 'string') {
            attrs.content = popover;
        }
        return Object.assign({}, attrs, popoverAttrs, (_a = {},
            _a[visibleKey] = innerVisibleRef.value,
            _a));
    }
    function delayHidePopover() {
        if (props.duration && innerVisibleRef.value) {
            // @ts-ignore
            timerIdRef.value = setTimeout(function () {
                innerVisibleRef.value = false;
            }, props.duration);
        }
    }
    watch(function () { return props.popoverVisible; }, function (val) {
        innerVisibleRef.value = val;
        if (!val && timerIdRef.value)
            clearTimeout(timerIdRef.value);
        if (val)
            delayHidePopover();
    });
    onMounted(function () {
        var popoverVisible = props.popoverVisible, scrollWrapper = props.scrollWrapper, scrollDebounce = props.scrollDebounce;
        innerVisibleRef.value = popoverVisible;
        if (scrollWrapper) {
            // 滚动元素, scroll elment
            var el = scrollWrapper === 'window'
                ? window
                : document.querySelector(scrollWrapper);
            if (el) {
                scrollElRef.value = el;
                debounceUpdatePopperRef.value = debounce(function () {
                    // @ts-ignore
                    var popover = _this.$refs.popover;
                    if (popover && typeof popover.updatePopper === 'function') {
                        popover.updatePopper();
                    }
                }, scrollDebounce);
                el.addEventListener('scroll', debounceUpdatePopperRef.value);
            }
        }
        delayHidePopover();
    });
    onUnmounted(function () {
        if (scrollElRef.value && debounceUpdatePopperRef.value) {
            scrollElRef.value.removeEventListener('scroll', debounceUpdatePopperRef.value);
        }
        if (timerIdRef.value)
            clearTimeout(timerIdRef.value);
        cleanup();
    });
    var innerPopoverAttrs = computed(function () { return genPopoverAttrs(className); });
    return {
        innerVisible: innerVisibleRef,
        genPopoverAttrs: genPopoverAttrs,
        innerPopoverAttrs: innerPopoverAttrs,
    };
}
